package com.simplegame.erp.bus.user.service.impl;


import java.sql.Timestamp;
import java.util.List;
import java.util.UUID;

import javax.annotation.Resource;

import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import com.simplegame.erp.bus.user.entity.Roles;
import com.simplegame.erp.bus.user.entity.User;
import com.simplegame.erp.bus.user.service.IUserService;
import com.simplegame.erp.bus.user.service.UserRepository;

/**
 *
 * @Author zeusgooogle@gmail.com
 * @sine   2015年9月23日 下午6:38:00
 *
 */
@Service
public class UserServiceImpl implements IUserService {

    @Resource
    private UserRepository repository;
    
    @Resource
    private PasswordEncoder passwordEncoder;
    
    @Override
    public boolean exist(String username) {
        User user = repository.findByUsername(username);
        
        return user != null;
    }

    @Override
    public User createUser(String username, String password) {
        User user = new User();
        
        String id = UUID.randomUUID().toString();
        id = id.replaceAll("\\-", "").toUpperCase();
        
        user.setId(id);
        user.setUsername(username);
        user.setPassword( passwordEncoder.encode(password) );
        user.setCreateTime(new Timestamp(System.currentTimeMillis()));
        
        repository.save(user);
        
        return user;
    }

    @Override
    public User findByUsername(String username) {
        return repository.findByUsername(username);
    }
    
    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        User user = repository.findByUsername(username);
        if(null == user) {
            return new org.springframework.security.core.userdetails.User(username, "", AuthorityUtils.NO_AUTHORITIES);
        }
        
        List<GrantedAuthority> auth = AuthorityUtils.commaSeparatedStringToAuthorityList(Roles.USER.name);
        if( user.getType() == Roles.ADMIN.type ) {
            auth = AuthorityUtils.createAuthorityList(Roles.ADMIN.name, Roles.USER.name);
        }
        String password = user.getPassword();
        
        return new org.springframework.security.core.userdetails.User(username, password, auth);
    }

    @Override
    public boolean isOnline() {
        String username = getOnlineUsername();
        return username != null;
    }
    
    @Override
    public String getOnlineUsername() {
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        
        if( auth.getPrincipal() instanceof org.springframework.security.core.userdetails.User ) {
            org.springframework.security.core.userdetails.User user = (org.springframework.security.core.userdetails.User)auth.getPrincipal();
            return user.getUsername();
        }
        
        return null;
    }

}
